﻿//	『昼夜逆転』工作室	http://jsdiy.web.fc2.com
//
//	FT232RLモジュール 7セグ4桁制御 サンプルアプリ
//	「時計」
//
//	2011/11/11	v1.00	初版

using System;
using System.Timers;	//Timer
using FT232_7Seg4Lib;

namespace ft232r7seg4clock
{
	class Program
	{
		private static SegCtrl segled;

		static void Main(string[] args)
		{
			PortMgr pm = new PortMgr();
			
			//ブレッドボードバージョンでのピンアサイン（白黒の回路図）
			//SRegPin.P1～P8とSegLedPin.A～DPの対応はデフォルト値の通りなのでSetSRegPort()は省略
			pm.SetSRegCtrl(FtPin.D0, FtPin.D4, FtPin.D2);
			pm.SetSegLedDigit(FtPin.D6, FtPin.D5, FtPin.D7, FtPin.D1);
			
			/*	基板バージョンでのピンアサイン（カラーの配線図）
			pm.SetSRegPort(SRegPin.P8, SegLedPin.A);
			pm.SetSRegPort(SRegPin.P1, SegLedPin.B);
			pm.SetSRegPort(SRegPin.P3, SegLedPin.C);
			pm.SetSRegPort(SRegPin.P5, SegLedPin.D);
			pm.SetSRegPort(SRegPin.P6, SegLedPin.E);
			pm.SetSRegPort(SRegPin.P7, SegLedPin.F);
			pm.SetSRegPort(SRegPin.P2, SegLedPin.G);
			pm.SetSRegPort(SRegPin.P4, SegLedPin.DP);
			pm.SetSRegCtrl(FtPin.D0, FtPin.D4, FtPin.D2);
			pm.SetSegLedDigit(FtPin.D6, FtPin.D5, FtPin.D7, FtPin.D1);
			*/
			segled = new SegCtrl(pm);
			
			//7セグ制御開始
			if (!segled.Start(true))
			{
				Console.WriteLine("FT232 ready...");
				if (!segled.Start()) AppClose(segled.EMessage, segled.ECode);
			}

			//説明表示
			Console.WriteLine("- 7セグ4桁時計 -  jsdiy/(C)2011~ http://jsdiy.web.fc2.com");
			Console.WriteLine();
			Console.WriteLine("キー操作" + Environment.NewLine +
				"[T]ime   時刻（時.分）" + Environment.NewLine +
				"[S]econd 時刻（分.秒）" + Environment.NewLine +
				"[D]ate   日付（月.日.）" + Environment.NewLine +
				"[W]eek   曜日" + Environment.NewLine +
				 Environment.NewLine +
				"時刻(T)表示中に[1][2][3]キーで時間制を変更できます。" + Environment.NewLine +
				"[1] 12時間制" + Environment.NewLine +
				"[2] 24時間制" + Environment.NewLine +
				"[3] 30時間制" + Environment.NewLine +
				 Environment.NewLine +
				"いつでも[Esc]キーでアプリを終了します。");

			//時計起動
			SSeg4Clock sclock = new SSeg4Clock(segled);
			sclock.Start();
			
			//アプリ終了
			if (segled.IsError)
				AppClose(segled.EMessage, segled.ECode);
			else
				AppClose();
		}

		//アプリ終了
		private static void AppClose() { AppClose(null, 0); }
		private static void AppClose(string mes, int exitCode)
		{
			//7セグ オブジェクト破棄
			if (segled != null) segled.Dispose();

			//終了メッセージを表示する
			if (mes != null)
			{
				Console.WriteLine(mes + Environment.NewLine);
				Console.WriteLine("Press any key.");
				Console.ReadKey();
			}

			Environment.Exit(exitCode);
		}

	}	//<- end of class

	class SSeg4Clock
	{
		enum EState { TIME, SECOND, DATE, WEEK };
		private static readonly string[] WEEKS = { "Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat." };
		
		private readonly SegCtrl segled;
		private readonly Timer timer;
		private static bool tci;
		
		//コンストラクタ
		public SSeg4Clock(SegCtrl segled)
		{
			this.segled = segled;
			tci = false;

			timer = new Timer(500);		//0.5秒
			timer.Elapsed += new ElapsedEventHandler(OnTimeElapsed);
		}
		
		//タイマー時間経過
		private static void OnTimeElapsed(object source, ElapsedEventArgs e)
		{
			tci = true;
		}
		
		//時計実行本体
		public void Start()
		{
			//タイマー開始
			tci = false;
			timer.Start();
			
			EState nowState = EState.TIME;
			bool isChanged = true;
			int hourMode = 24;	//12,24,30時間制
			DateTime dt;
			bool mainLoop = true;
			
			while (mainLoop)
			{
				//キー入力判定
				if (Console.KeyAvailable)
				{
					ConsoleKeyInfo keyInfo = Console.ReadKey(true);
					while (Console.KeyAvailable) Console.ReadKey(true);		//キーバッファクリア（連打対策）
					
					EState prevState = nowState;
					switch (keyInfo.Key)
					{
					case ConsoleKey.T: nowState = EState.TIME; break;
					case ConsoleKey.S: nowState = EState.SECOND; break;
					case ConsoleKey.D: nowState = EState.DATE; break;
					case ConsoleKey.W: nowState = EState.WEEK; break;
					case ConsoleKey.Escape: mainLoop = false;	continue;	//唯一、メインループを抜ける処理
					default:	break;
					}
					isChanged = (prevState != nowState);

					//時間制変更
					if (nowState == EState.TIME)
					{
						int prevHourMode = hourMode;
						if (keyInfo.KeyChar == '1') hourMode = 12;
						if (keyInfo.KeyChar == '2') hourMode = 24;
						if (keyInfo.KeyChar == '3') hourMode = 30;
						isChanged = isChanged || (prevHourMode != hourMode);
					}
				}

				//動作状態
				switch (nowState)
				{
				case EState.TIME:	//Time (HH.MM) HHは空白詰め2桁, MMは0詰め2桁
					if (!isChanged && !tci) break;
					tci = false;
					dt = DateTime.Now;
					if (isChanged || dt.Second == 0)
					{
						int hour = dt.Hour;		//24時間制
						if (hourMode == 12 && 12 < hour) hour -= 12;	//12時間制
						if (hourMode == 30 && hour < 6) hour += 24;		//30時間制
						segled.PutString(string.Format("{0,2:D}{1:mm}", hour, dt), 0);
					}
					segled.PutSegDP(dt.Millisecond < 500, 1);	//点滅
					break;

				case EState.SECOND:	//Second (MM.SS) ともに0詰め2桁
					if (!isChanged && !tci) break;
					tci = false;
					dt = DateTime.Now;
					if (isChanged || dt.Millisecond < 500)
						segled.PutString(string.Format("{0:mm}.{0:ss}", dt), 0, true);
					break;

				case EState.DATE:	//Date (MM.DD.) ともに空白詰め2桁
					if (!isChanged) break;
					dt = DateTime.Now;
					segled.PutString(string.Format("{0,2:D}.{1,2:D}.", dt.Month, dt.Day), 0);
					break;

				case EState.WEEK:	//Week
					if (!isChanged) break;
					dt = DateTime.Now;
					segled.PutString(string.Format("{0,4}", WEEKS[(int)dt.DayOfWeek]), 0);
					break;

				default:	break;
				}
				
				isChanged = false;

			}	//<- end of while()

			//タイマー停止／破棄
			timer.Stop();
			timer.Dispose();
		}
		
	}	//<- end of class

}	//<- end of namespace
